package cordova.plugins;

//VIATOM imports
import com.viatom.azur.bluetooth.BTConnectListener;
import com.viatom.azur.bluetooth.BTUtils;
import com.viatom.azur.bluetooth.GetInfoThreadListener;
import com.viatom.azur.bluetooth.PingThreadListener;
import com.viatom.azur.bluetooth.ReadFileListener;
import com.viatom.azur.measurement.ECGInnerItem;
import com.viatom.azur.measurement.ECGItem;
import com.viatom.azur.measurement.SPO2Item;
import com.viatom.azur.measurement.TempItem;
import com.viatom.azur.utils.FileDriver;
import com.viatom.azur.utils.FileUtils;
import com.viatom.azur.utils.LogUtils;

//Cordova imports
import org.apache.cordova.CordovaWebView;
import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.PluginResult;
import org.apache.cordova.CordovaInterface;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

//ANDROID OS imports
import android.content.Intent;
import android.app.Service;
import android.os.Bundle;
import android.content.ComponentName;
import android.os.IBinder;
import android.os.Binder;
import android.content.Context;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.util.Log;
import android.app.Activity;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.os.Environment;

//SE 1.6 imports
import java.util.List;
import  java.lang.InterruptedException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.io.File;
import java.nio.charset.StandardCharsets;
import java.util.Stack;


public class checkMeInterface extends CordovaPlugin{

    private CordovaInterface cordova;
    private CordovaWebView webView;
    private CallbackContext CP_CALLBACK;

    @Override
    public boolean execute(String action, JSONArray args, CallbackContext callbackContext) throws JSONException {
      if(action.equals("COMMAND_BT_TRANSFER")){
            this.CP_CALLBACK = callbackContext;
            PluginResult r = new PluginResult(PluginResult.Status.NO_RESULT);
            r.setKeepCallback(true);
            callbackContext.sendPluginResult(r);

            Intent i = new Intent(this.cordova.getActivity().getApplicationContext(), BTActivity.class);
            i.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
            cordova.startActivityForResult(this,i,90);

            return true;
      }else{
            return false;
      }
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent intent){

        String result = "";

        this.CP_CALLBACK.sendPluginResult(new PluginResult(PluginResult.Status.OK, result));

    // when there is no direct result form your execute-method use sendPluginResult because most plugins I saw and made recently (Reminder) prefer sendPluginResult to success/error
    // this.callbackContext.success(result.toString());
    }


    public class BTActivity extends Activity implements BTConnectListener, ServiceConnection, PingThreadListener, GetInfoThreadListener{


        private LinkedList<String> objectives = new LinkedList();
        private Intent outputIntent = new Intent();
        @Override
        protected void onCreate(Bundle savedInstanceState) {
            super.onCreate(savedInstanceState);
            Intent i = new Intent(this, BTUtils.class);
            bindService(i, this, Service.BIND_AUTO_CREATE);
            objectives.add("ecg.dat");
            objectives.add("oxi.dat");
            objectives.add("tmp.dat");
            objectives.add("bpcal.dat");
            //objectives.add("slm.dat");
        }

        public class Worker implements Runnable{

            private BTUtils.BTBinder mBinder;
            private String objective;

            public Worker(BTUtils.BTBinder binder, String objective){
                mBinder = binder;
                this.objective = objective;
            }

            public void setBinder(BTUtils.BTBinder binder){
                mBinder = binder;
            }

            public void setObjective(String objective){
                this.objective = objective;
            }

            @Override
            public void run(){
                mBinder.interfaceReadFile(objective, (byte)0,5000, new ReadFileListenerImpl());
                while(mBinder.isAnyThreadRunning()){
                    if(!mBinder.isAnyThreadRunning()){
                        break;
                    }
                }
            }

        }

        public class ReadFileListenerImpl implements ReadFileListener{
            @Override
            public void onReadPartFinished(String fileName, byte fileType, float percentage){
                LogUtils.d(fileName + "Read " + percentage*100 + "%");
            }
            @Override
            public void onReadSuccess(String fileName, byte fileType, byte[] fileBuf){
                String location = Environment.getExternalStorageDirectory().toString() + "/CheckMeIM";
                File f = new File(location);
                if(!f.exists()){
                    f.mkdirs();
                }
                //FileDriver.write(f , fileName, fileBuf);
                FileUtils.saveListToFileWithoutUndownloaded(f, fileName, null);
            }
            @Override
            public void onReadFailed(String fileName, byte fileType, byte errCode){
                Log.e("ERROR",fileName + " : Read Failed - No measurement detected, proceeding if there are any objectives left.");
            }
        }

        private BTUtils.BTBinder mBinder = null;
        @Override
        public void onGetInfoFailed(byte errCode){
            Log.e("ERR", "INFO GET FAILED");
        }

        @Override
        public void onGetInfoSuccess(String info){
            tesRead();
        }

        @Override
        public void onPingFailed(byte errCode){
        }

        @Override
        public void onPingSuccess(){
            mBinder.interfaceGetInfo(10000, this);
        }

        @Override
        public void onConnectFailed(byte errCode){
            Log.e("ERR", "CONNECTION FAILED : " + errCode);
        }

        @Override
        public void onConnectSuccess(){
            Log.e("CHECKME" , "CONNECT SUCCESS");
            mBinder.interfacePing(1000, this);
        }

        @Override
        public void onServiceConnected(ComponentName componentName, IBinder service) {
            mBinder = (BTUtils.BTBinder) service;
            Log.d("BT BIND SERVICE","BOUND");
            testConnectCheckme("B0:B4:48:94:99:41");
        }

        @Override
        public void onServiceDisconnected(ComponentName componentName) {
        }

        public void testConnectCheckme(String MAC){
            mBinder.interfaceConnect(MAC,this);
        }


       public void tesRead()
       {
           //mBinder.interfaceReadFile("ecg.dat", (byte)0,5000, this);
           //mBinder.interfaceReadFile(objectives.get(0), (byte)0,5000, new ReadFileListenerImpl());

           Worker workerECG = new Worker(mBinder, "ecg.dat");
           Thread t = new Thread(workerECG);
           t.start();
           try{
               t.join();
               Worker workerOXI = new Worker(mBinder, "oxi.dat");
               t = new Thread(workerOXI);
               t.start();

               t.join();
               Worker workerTMP = new Worker(mBinder, "tmp.dat");
               t = new Thread(workerTMP);
               t.start();

               t.join();
               Worker workerBPCAL = new Worker(mBinder, "bpcal.dat");
               t = new Thread(workerBPCAL);
               t.start();

               t.join();
               Worker workerSLM = new Worker(mBinder, "slm.dat");
               t = new Thread(workerSLM);
               t.start();

          }catch (InterruptedException e){

           }

           String location = Environment.getExternalStorageDirectory().toString() + "/CheckMeIM";

    //       List<ECGItem> ECGItems = FileUtils.readECGList(new File(location), "ecg.dat");
    //        if(ECGItems != null && ECGItems.size() != 0){
    //            ECGItem item = ECGItems.get(0);
    //            String fileName = new SimpleDateFormat("yyyyMMddHHmmss").format(item.getDate());
    //            ECGInnerItem innerItem = FileUtils.readECGInnerItem(new File(location), fileName);
    //            item.setInnerItem(innerItem);
    //            Log.e("DEBUG", "HR = "+innerItem.getHR());
    //        }


            JSONObject outputObjectJSON = new JSONObject();
            JSONArray SPO2Container = new JSONArray();


        try{
           Log.e("OXI ITEM", "TRYING TO READ OXI VALUES");
           List<SPO2Item> SPO2Items = FileUtils.readSPO2List(new File(location), "oxi.dat");
           if(SPO2Items != null && SPO2Items.size() != 0){
               for(int i = 0; i < SPO2Items.size(); i++){

                    JSONObject spo2Item = new JSONObject();

                    spo2Item.put("OXI_PERCENTAGE" , SPO2Items.get(i).getOxygen()+"");
                    Log.e("OXI ITEM", SPO2Items.get(i).getOxygen() + " OXI");
                    spo2Item.put("OXI_PI" , SPO2Items.get(i).getPi()+"");
                    Log.e("OXI ITEM", SPO2Items.get(i).getPi()+" PI");
                    spo2Item.put("OXI_PR" , SPO2Items.get(i).getPr()+"");
                    Log.e("OXI ITEM", SPO2Items.get(i).getPr()+" PR");

                    SPO2Container.put(spo2Item);
               }
           }

           outputObjectJSON.put("SPO2", SPO2Container);
           outputIntent.putExtra("CHECKME_DATA", outputObjectJSON.toString());
           finishIntent();

        }
        catch(JSONException e){
                Log.e("ERROR","Error writing json");
        }

       }

        public void writeFile(){
            String filename = "myfile";
            String string = "Hello world!";
            FileOutputStream outputStream;

            try {
                outputStream = openFileOutput(filename, Context.MODE_APPEND);
                outputStream.write(string.getBytes());
                outputStream.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        public void testReadFile(){
            //TODO
        }

        public ECGItem testDecodeECGItem(byte[] buf){
            if(buf != null){
                ECGItem item = new ECGItem(buf);
                Log.e("DEBUG ",item.getInnerItem() + "");
                return item;
            }
            return null;
        }

        public void finishIntent(){
            setResult(RESULT_OK, outputIntent);
            finish();
        }
    }
}